#include <msp/io/memory.h>
#include "buffer.h"
#include "waveform.h"

using namespace std;

namespace Msp {
namespace AL {

Buffer::Buffer()
{
	alGenBuffers(1, &id);
}

Buffer::~Buffer()
{
	alDeleteBuffers(1, &id);
}

void Buffer::data(Format fmt, const void *dt, sizei size, sizei freq)
{
	alBufferData(id, fmt, dt, size, freq);
}

void Buffer::data(const Waveform &wave)
{
	data(wave.get_format(), wave.get_data(), wave.get_size(), wave.get_frequency());
}

void Buffer::load_data(const string &fn)
{
	Waveform wave;
	wave.load_file(fn);

	data(wave);
}


Buffer::Loader::Loader(Buffer &b):
	buf(b)
{
	add("sound_data", &Loader::sound_data);
}

void Buffer::Loader::sound_data(const string &data)
{
	Waveform wave;
	IO::Memory mem(data.data(), data.size());
	wave.load_io(mem);

	buf.data(wave);
}

} // namespace AL
} // namespace Msp
