#ifndef MSP_AL_SOUNDDECODER_H_
#define MSP_AL_SOUNDDECODER_H_

#include <string>
#include <msp/io/seekable.h>
#include "format.h"

namespace Msp {
namespace AL {

class unsupported_sound: public std::runtime_error
{
public:
	unsupported_sound(const std::string &);
	virtual ~unsupported_sound() throw() { }
};


/**
This class facilitates loading sound files.  Currently only Ogg Vorbis is
supported.
*/
class SoundDecoder
{
private:
	IO::Seekable *source;
protected:
	unsigned freq;
	unsigned size;
	Format format;
	bool eof_flag;

	SoundDecoder();
public:
	virtual ~SoundDecoder();

	static SoundDecoder *open_file(const std::string &);
	static SoundDecoder *open_io(IO::Seekable &);

	virtual void rewind() = 0;
	virtual unsigned read(char *, unsigned) = 0;
	bool eof() const { return eof_flag; }

	Format get_format() const { return format; }
	unsigned get_frequency() const { return freq; }
	unsigned get_size() const { return size; }
};

} // namespace AL
} // namespace Msp

#endif
